#!/usr/bin/perl -w

# admin/index.cgi
#
# This script is part of Birch. It displays the administrator interface, and calls other scripts to
# create, edit, and delete posts.
#
# Copyright (c) 2006, Ross Harmes
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
# 
#	*	Redistributions of source code must retain the above copyright notice, 
#		this list of conditions and the following disclaimer.
#	*	Redistributions in binary form must reproduce the above copyright notice, 
#		this list of conditions and the following disclaimer in the documentation 
#		and/or other materials provided with the distribution.
#	*	Neither the name of View Software nor the names of its contributors may be 
#		used to endorse or promote products derived from this software without 
#		specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Changelog: 
#	2004.05.23: File created.
#	2005.12.11: Added custom photo sizes.

use CGI;
use strict;

my $scriptURL = CGI::url();  # the http url of the script 

my %vars = CGI::Vars();      # use CGI.pm to grab CGI parameters

print "Content-type: text/html\n\n";

# Get the data for the Edit Posts section.

my $photo_list = "../cgi-bin/photos.xml"; # This is the XML file with the dates & captions of the posted photos.

my ($line_num, $list_posts, $select_posts, $num_of_posts, $caption, $dimensions);
my @line;

# Open the photo list and read the lines into an array.

open(LIST, "< $photo_list") or $list_posts = "Unable to open file: " . $!;
while(<LIST>) { $line[$line_num++] = $_; }
close LIST;
	
$num_of_posts = $line_num - 1;
	
# Lines 4 through 6 are the latest 3 posts.
	
for($line_num = 3; $line_num < 6; $line_num++) {
	if($line[$line_num] =~ /<date>(.*?)<\/date><caption>(.*?)<\/caption><orientation>(.*?)<\/orientation>/) { 
		if($3 eq 'horizontal') { $dimensions = "width=\"66\" height=\"50\""; }
		else { $dimensions = "width=\"50\" height=\"66\" class=\"vert\""; }
	
		if($line_num == 3) {
			$list_posts .= "\t\t\t<div class=\"post-to-edit\">\n" .
								"\t\t\t\t<input type=\"hidden\" name=\"post1_date\" value=\"" . $1 . "\" />\n" .
						   		"\t\t\t\t<img src=\"/thumbnails/" . $1 . ".jpg\" $dimensions alt=\"\" />\n" . 
						   		"\t\t\t\t<textarea class=\"caption-box\" name=\"post1\">" . $2 . "</textarea>\n"; 
			}
		elsif($line_num == 4) {
			$list_posts .= "\t\t\t</div>\n" .
								"\t\t\t<div class=\"post-to-edit\">\n" .
								"\t\t\t\t<input type=\"hidden\" name=\"post2_date\" value=\"" . $1 . "\" />\n" .
								"\t\t\t\t<img src=\"/thumbnails/" . $1 . ".jpg\" $dimensions alt=\"\" />\n" . 
								"\t\t\t\t<textarea class=\"caption-box\" name=\"post2\">" . $2 . "</textarea>\n"; 
			}
		else {
			$list_posts .= "\t\t\t</div>\n" .
								"\t\t\t<div class=\"post-to-edit\">\n" . 
								"\t\t\t\t<input type=\"hidden\" name=\"post3_date\" value=\"" . $1 . "\" />\n" .
								"\t\t\t\t<img src=\"/thumbnails/" . $1 . ".jpg\" $dimensions alt=\"\" />\n" . 
								"\t\t\t\t<textarea class=\"caption-box\" name=\"post3\">" . $2 . "</textarea>\n";
			}
		}
	}

if($list_posts ne '') { $list_posts .= "\t\t\t\t<input type=\"submit\" value=\"Submit Changes\" class=\"submit\" />\n\t\t\t</div>"; }
	
# List all of the posts in the select menu.

for($line_num = 3; $line_num < $num_of_posts; $line_num++) {
	if($line[$line_num] =~ /<date>(.*?)<\/date><caption>(.*?)<\/caption>/) { 
		$caption = $2;
		if(length($caption) > 65) { $caption = substr($caption, 0, 65) . "..." }
		$select_posts .= "\t\t\t\t\t<option>" . $1 . " :: " . $caption . "</option>\n"
		}
	}

# Find the current template.

my $stylesheet = "../includes/styles.css";
my $current_template;

open STYLESHEET, "< $stylesheet" or return;
$current_template = <STYLESHEET>;
close STYLESHEET;

if($current_template =~ /\/\*\*\* (.*?) \*\*\*\//) { $current_template = $1; }
else { $current_template = "Unnamed"; }

# Find other available templates.

my $stylesheet_location = "../includes/";
my ($stylesheet_options, $name);

opendir INCLUDES, "$stylesheet_location" or return;
while($name = readdir INCLUDES) { 
	next if($name eq "." or 
			$name eq ".." or 
			$name eq "styles.css" or
			$name eq ".DS_Store" or
			$name eq "sidebar.html" or 
			$name eq "title.html" or 
			$name eq "author.html" or
			$name eq "admin.css" or 
			$name eq "admin.js");
	$name =~ s/_/ /g;
	$name =~ s/\.css//g;		
	$stylesheet_options .= "\t\t\t\t\t<option>" . $name . "</option>\n"; 
	}
closedir INCLUDES;

# Get the current text of the colophon.

my $colophon = "../colophon.html";
my $colophon_text;

open COLOPHON_RD, "< $colophon" or return;
while(<COLOPHON_RD>) { $colophon_text .= $_; }
close COLOPHON_RD;

if($colophon_text =~ /<div id="colophon">\n\n(.*)<\/div>\n<\/div>\n\n<\/body>/is) { $colophon_text = $1; }


# Print the XHTML.

print <<END_OF_HTML
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd  ">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"> 
<head>
	<title>birch administration</title>

	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />

	<link rel="stylesheet" type="text/css" href="/includes/admin.css" />
	<script type="text/javascript" src="/includes/admin.js"></script> 
	
</head>
<body>

<div id="content">

	<h1 id="title">Birch Administration :: <a href="/">Main</a></h1>
	
	<a href="#new-post" onclick="viewSection('new-post', 'new-post-tab'); return false;" class="nav selected" id="new-post-tab">New Post</a>
	<a href="#edit-post" onclick="viewSection('edit-post', 'edit-post-tab'); return false;" class="nav" id="edit-post-tab">Edit Post</a>
	<a href="#options" 	onclick="viewSection('options', 'options-tab'); return false;" class="nav" id="options-tab">Options</a>
	

	<div id="new-post" class="section">
	
		<form action="/cgi-bin/upload.cgi" method="post" enctype="multipart/form-data" name="stage1">
		
			<div id="stage-1">

				<input type="hidden" name="stage" value="1" />

				<strong>Step 1:</strong>
				<label for="photo" id="photo-label">Upload Photo</label>
				<input type="file" name="photo" id="photo" />
				
				<label for="photo-width">Width:</label>
				<input type="text" name="photo-width" id="photo-width" value="600" />
				
				<label for="photo-height">Height:</label>
				<input type="text" name="photo-height" id="photo-height" value="450" />

			</div>
	
			<div id="stage-2">

				<strong>Step 2:</strong>
				<label for="thumb">Upload Thumbnail</label>
				<input type="file" name="thumb" id="thumb" /><br />
				
				<p>
					For horizontal photos: the thumbnail should have a width of 66 pixels and a height proportional to that width.<br />
					For vertical photos: the thumbnail should have a height of 50 pixels and a width proportional to that height.<br />
				</p>

			</div>
			
			<div id="stage-options">
			
				<strong>Orientation:</strong>
				<input type="radio" name="orientation" value="horizontal" id="horizontal" checked="checked" /> 
				<label for="horizontal" id="horizontal-label">Horizontal</label>
				<input type="radio" name="orientation" value="vertical" id="vertical" /> 
				<label for="vertical" id="vertical-label">Vertical</label>
			
				<input type="submit" value="Upload" class="submit" />
			
			</div>
		
		</form>
		
	</div>
	
	
	
	<div id="edit-post" class="section" style="display : none;">
	
		<form action="/cgi-bin/edit_post.cgi" method="post">
		
			<input type="hidden" name="type" value="change" />
				 
$list_posts
	 		
	 	</form>
	 	<form action="/cgi-bin/edit_post.cgi" method="post">
	 		
	 		<input type="hidden" name="type" value="open" />
	 		
	 		<div class="post-to-edit">
	 			<select name="post-select" id="post-select">
					<option selected>Choose Post to Edit   </option>
	 				<option>----</option>
$select_posts
	 			</select>
	 		
	 			<input type="submit" value="Edit Post" />
	 		</div>
	 	
	 	</form>
	 		
	</div>
	
	
	
	<div id="options" class="section" style="display : none;">
	
		<div id="change-template">
			<form action="/cgi-bin/options.cgi" method="post">
				<input type="hidden" name="change" value="template" />
				
				<h2>Change Template</h2>
			
				Current Template: $current_template
				
				<input type="submit" value="Change Template" />
				
				<select id="new-stylesheet" name="new_stylesheet">
					<option selected>Select New Template</option>
					<option>----</option>
$stylesheet_options
	 			</select>
			</form>
		</div>
		
		<div id="change-colophon">
			<form action="/cgi-bin/options.cgi" method="post">
				<input type="hidden" name="change" value="colophon" />
				
				<h2>Change Colophon</h2>
			
				<textarea id="colophon-text" name="colophon_text" >$colophon_text
				</textarea>
				<br />
	 			<input type="submit" class="submit" value="Change Colophon" />
			</form>		
		</div>
		
		<div id="change-password">
			<form action="/cgi-bin/options.cgi" method="post" name="newPasswordForm" onsubmit="return checkPasswords('new_pass', 'new_pass_2');">
				<input type="hidden" name="change" value="password" />
				
				<h2>Change Username &amp; Password</h2>
			
				<label for="current_user">Current Username:</label>
				<input type="text" name="current_user" id="current_user" class="change-username" value="" />
				
				<label for="current_pass">Current Password:</label>
				<input type="password" name="current_pass" id="current_pass" class="change-username" />
				
				<div class="bar"></div>
				
				<label for="new_user">New Username:</label>
				<input type="text" name="new_user" id="new_user" class="change-username" value="" />
				
				<label for="new_pass">New Password:</label>
				<input type="password" name="new_pass" id="new_pass" class="change-username" />
				
				<label for="new_pass_2">Verify Password:</label>
				<input type="password" name="new_pass_2" id="new_pass_2" class="change-username" />
	 			
	 			<div class="bar"></div>
	 			
	 			<input type="submit" class="submit" id="password-submit" value="Change Login Info" />
			</form>		
		</div>

		<br class="clear" />

	</div>


</div>

</body>
</html>

END_OF_HTML
;

exit;



