#!/usr/bin/perl -w

# archives/index.cgi
#
# This script is part of Birch. It displays the archives, both recent and by date.
#
# Copyright (c) 2006, Ross Harmes
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
# 
#	*	Redistributions of source code must retain the above copyright notice, 
#		this list of conditions and the following disclaimer.
#	*	Redistributions in binary form must reproduce the above copyright notice, 
#		this list of conditions and the following disclaimer in the documentation 
#		and/or other materials provided with the distribution.
#	*	Neither the name of View Software nor the names of its contributors may be 
#		used to endorse or promote products derived from this software without 
#		specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Changelog: 
#	2005.03.15: File created.
#	2005.12.11: Added custom photo sizes.


use POSIX;
use strict;
use CGI qw(:standard);

my $photo_list 	 = "../cgi-bin/photos.xml"; # This is the XML file with the dates & captions of the posted photos.
my $title_file 	 = "../includes/title.html";
my $sidebar_file = "../includes/sidebar.html";

my $date = param("date");

my ($page_title, $sidebar, $file_line, $key, $first, $dimensions, $height, $width, $dimensions_thumb, $older_photos, $html_to_print);
my (@archive, @line);
my (%dates, %years);

my %month = ("00" => "", "01" => "January", "02" => "February", "03" => "March", "04" => "April", "05" => "May", "06" => "June", "07" => "July", "08" => "August", "09" => "September", "10" => "October", "11" => "November", "12" => "December");


# Get the contents of the title and sidebar fragments.

open TITLE, $title_file or die "Can't open $title_file for reading: $!";
while ($file_line = <TITLE>) { $page_title .= $file_line; }
close TITLE;

open SIDEBAR, $sidebar_file or die "Can't open $sidebar_file for reading: $!";
while ($file_line = <SIDEBAR>) { $sidebar .= $file_line; }
close SIDEBAR;

# Open the photo list and read the lines into an array in reverse order.

open(LIST, "< $photo_list");
@line = <LIST>;
close LIST;

# Process the array and display the page based on $date.

if($date eq 'all') {

	# Process the array and display all photos.

	for(my $n = 3; $n < scalar(@line); $n++) {
		if($line[$n] =~ /<date>(.*?)<\/date><caption>.*?<\/caption><orientation>(.*?)<\/orientation><width>(.*?)<\/width><height>(.*?)<\/height>/) {
	
			if($2 eq 'horizontal') { 
				$height = ceil((66 * $4) / $3);
				$dimensions_thumb = "width=\"66\" height=\"" . $height . "\""; 
				}
			else { 
				$width = ceil((50 * $3) / $4);
				$dimensions_thumb = "width=\"" . $width . "\" height=\"50\" class=\"vert\""; 
				}
		
			$older_photos .= "\t\t<a href=\"/cgi-bin/frame.cgi?date=" . $1 . "\"><img src=\"/thumbnails/" . $1 . ".jpg\" $dimensions_thumb alt=\"\" /></a>";
			}
		}
	
	# Format the results into a single string.
	
	$html_to_print = <<END_OF_HTML	
	<div id="recent">
		<h3>All Photos</h3>

$older_photos
	</div>
	 
END_OF_HTML
;
	}
	
	
elsif($date =~ /(\d\d\d\d)(\d\d)/) {

	# Process the array for the month section.
	
	my $y = $1;
	my $m = $2;
	
	# Process the array for the date section.

	for(my $n = 3; $n < scalar(@line); $n++) {
		if($line[$n] =~ /<date>(\d\d\d\d)_(\d\d)_(\d\d)_(\d\d)_(\d\d)_(\d\d)<\/date><caption>.*?<\/caption><orientation>(.*?)<\/orientation><width>(.*?)<\/width><height>(.*?)<\/height>/) {	
			if(($1 eq $y) and ($2 eq $m)) {
			
				if($7 eq 'horizontal') { 
				$height = ceil((66 * $9) / $8);
				$dimensions_thumb = "width=\"66\" height=\"" . $height . "\""; 
				}
			else { 
				$width = ceil((50 * $8) / $9);
				$dimensions_thumb = "width=\"" . $width . "\" height=\"50\" class=\"vert\""; 
				}
				
				my $date_full = $1 . "_" . $2 . "_" . $3 . "_" . $4 . "_" . $5 . "_" . $6;
				$older_photos .= "\t\t<a href=\"/cgi-bin/frame.cgi?date=" . $date_full . "\"><img src=\"/thumbnails/" . $date_full . ".jpg\" $dimensions_thumb alt=\"\" /></a>";
				}
			}
		}
	
	# Format the results into a single string.
	
	$html_to_print = <<END_OF_HTML	
	<div id="recent">
		<h3>Photos from $month{$m}, $y</h3>

$older_photos
	</div>
	 
END_OF_HTML
;
	}
	
	
else {

	# Process the array for the recent section.

	for(my $n = 3; $n < 16; $n++) {
		if($line[$n] =~ /<date>(.*?)<\/date><caption>.*?<\/caption><orientation>(.*?)<\/orientation><width>(.*?)<\/width><height>(.*?)<\/height>/) {
	
			if($2 eq 'horizontal') { 
				$height = ceil((66 * $4) / $3);
				$dimensions_thumb = "width=\"66\" height=\"" . $height . "\""; 
				}
			else { 
				$width = ceil((50 * $3) / $4);
				$dimensions_thumb = "width=\"" . $width . "\" height=\"50\" class=\"vert\""; 
				}
		
			$archive[$n - 3] = "\t\t<a href=\"/cgi-bin/frame.cgi?date=" . $1 . "\"><img src=\"/thumbnails/" . $1 . ".jpg\" $dimensions_thumb alt=\"\" /></a>";
			}
		}
	
	# Process the array for the date section.

	for(my $n = 1; $n < scalar(@line); $n++) {
		if($line[$n] =~ /<date>(\d\d\d\d)_(\d\d)_(\d\d)_(\d\d)_(\d\d)_(\d\d)<\/date><caption>.*?<\/caption><orientation>.*?<\/orientation>/) {	
			$dates{$1 . $2}++;
			}
		}

	$first = 1;

	foreach $key (sort reverse_numeric keys %dates) {
		$key =~ /(\d\d\d\d)(\d\d)/;
		my $y = $1;
		my $m = $2;
	
		if($years{$y} != 1) { 
		
			if($first == 1) { $first = 0; }
			else			{ $older_photos .= "\t\t</ul>\n\n"; }
			
			$older_photos .= "\t\t<h4>" . $y . "</h4>\n\t\t<ul>\n"; 
			$years{$y} = 1;
			}
	
		$older_photos .= "\t\t\t<li><a href=\"/archives/?date=" . $key . "\">" . $month{$m} . "</a>";

		for(my $n = 0; $n < $dates{$key}; $n++) { 
			
			if($n == 20) {
				$older_photos .= "<span class=\"plus\">+</span>";
				last;
				}
			else {
				$older_photos .= "<span></span>";
				}
			}
	
		$older_photos .= "</li>\n";
		}
	$older_photos .= "\t\t</ul>\n\n";
	
	# Format the results into a single string.
	
	$html_to_print = <<END_OF_HTML	
	<div id="recent">
		<h3>Recent Photos</h3>
$archive[0]
$archive[1]
$archive[2]
$archive[3]
$archive[4]
$archive[5]
$archive[6]
$archive[7]
$archive[8]
$archive[9]
$archive[10]
$archive[11]
	</div>
	
	<div id="dates">
		<h3>Older Photos</h3>
		
$older_photos
		
		<h4><a href="/archives/?date=all">Show All</a></h4>
	</div> 
END_OF_HTML
;
	}


# Print the page.

print <<END_OF_HTML
Content-type: text/html

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd  ">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"> 
<head>
	<title>$page_title :: Archives</title>

	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<meta name="author" content="" />
	<meta name="keywords" content="" />
	<meta name="description" content="" />
	<meta name="robots" content="index" />
		
	<link rel="stylesheet" type="text/css" href="/includes/styles.css" />
	
</head>
<body>

<div id="content">
<div id="sidebar">

$sidebar

</div>

<div id="main">

$html_to_print
	
</div>
</div>

</body>
</html>

END_OF_HTML
;



### Subroutines: ###

# Reverse numberic sort subroutine (expecting $a and $b).

sub reverse_numeric { $b <=> $a } # Identical to { if($a < $b) { 1 } elsif($a > $b) { -1 } else { 0 } }