#!/usr/bin/perl -w

# photoblog.cgi
#
# This script is part of Birch. It accesses a file where the names and captions for photos are stored,
# displays them in XHTML and provides navigation.
#
# Call with <!--#exec cgi="/cgi-bin/photoblog.cgi"--> where you want the contents to appear.
#
# Copyright (c) 2006, Ross Harmes
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
# 
#	*	Redistributions of source code must retain the above copyright notice, 
#		this list of conditions and the following disclaimer.
#	*	Redistributions in binary form must reproduce the above copyright notice, 
#		this list of conditions and the following disclaimer in the documentation 
#		and/or other materials provided with the distribution.
#	*	Neither the name of View Software nor the names of its contributors may be 
#		used to endorse or promote products derived from this software without 
#		specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Changelog: 
#	2004.05.21: File created.
#	2004.06.26: Added the ability to have a vertically oriented photo.
#	2005.03.19: Added navigation arrows overlaying the current photo.
#	2005.12.11: Added custom photo sizes.

use POSIX;
use lib '.';
use Birch;
use strict;

my $photo_list = "photos.xml"; # This is the XML file with the dates & captions of the posted photos.

my ($current_photo, $current_caption, $line_num, $orientation, $width, $height, $left_arrow, $left_photo_arrow);
my $dimensions;
my $dimensions_thumb;
my (@archive, @dimensions);
my @line;
my %current_date;

# Open the photo list and read the first 9 lines into an array.

open(LIST, "< $photo_list");
for($line_num = 0; $line_num < 9; $line_num++) { $line[$line_num] = <LIST>; }
close LIST;
	
# The fourth line is the latest post.
	
if($line[3] =~ /<date>(.*?)<\/date><caption>(.*?)<\/caption><orientation>(.*?)<\/orientation><width>(.*?)<\/width><height>(.*?)<\/height>/) {
	$current_photo = $1;
	$current_caption = $2;
	$orientation = $3;
	$width = $4;
	$height = $5;
	
	$dimensions = "width=\"" . $width . "\" height=\"" . $height . "\""
	
#	if($orientation eq 'horizontal') { $dimensions = "width=\"600\" height=\"450\"" }
#	else { $dimensions = "width=\"450\" height=\"600\" class=\"vert\"" }
	}
else { exit; } # If there aren't any photos, don't display anything.
	
# Lines 5 through 9 are the latest 5 posts.
	
for($line_num = 4; $line_num < 9; $line_num++) {
	
	if($line[$line_num] =~ /<date>(.*?)<\/date><caption>.*?<\/caption><orientation>(.*?)<\/orientation><width>(.*?)<\/width><height>(.*?)<\/height>/) { 
		$archive[8 - $line_num] = "<a href=\"/cgi-bin/frame.cgi?date=" . $1 . "\"><img src=\"thumbnails/" . $1 . ".jpg\" "; 
		
		if($2 eq 'horizontal') { 
			$height = ceil((66 * $4) / $3);
			$archive[8 - $line_num] .= "width=\"66\" height=\"" . $height . "\" alt=\"\" /></a>"; 
			}
		else { 
			$width = ceil((50 * $3) / $4);
			$archive[8 - $line_num] .= "width=\"" . $width . "\" height=\"50\" class=\"vert\" alt=\"\" /></a>"; 
			}
		
		if(8 - $line_num == 4) { 
			$left_arrow = "<a href=\"/cgi-bin/frame.cgi?date=" . $1 . "\" id=\"arrow-left\"></a>"; 
			$left_photo_arrow = "<a href=\"/cgi-bin/frame.cgi?date=" . $1 . "\" id=\"photo-arrow-left\"></a>";
			}
		}
	else { 
		$archive[8 - $line_num] = ''; 
		if(8 - $line_num == 4) { $left_arrow = ''; }
		}
	}
	
# Filter out the date and time.

if($current_photo =~ /(\d\d\d\d)_(\d\d)_(\d\d)_(\d\d)_(\d\d)_(\d\d)/) {
	$current_date{'year'} = 	$1;
	$current_date{'month'} = 	$2;
	$current_date{'day'} = 		$3;
	$current_date{'hour'} = 	$4;
	$current_date{'minute'} = 	$5;
	$current_date{'second'} = 	$6;
	
	$current_date{'serial'} = $1 . $2 . $3 . $4 . $5 . $6;
	}
	
# Print the XHTML. 

print <<END_OF_HTML
Content-type: text/html

<!-- Begin output of photoblog.cgi -->

	<div id="photo">
		$left_photo_arrow
		<img src="photos/$current_photo.jpg" $dimensions alt="" />
	</div>
		
	<div id="caption">
		<div id="caption-title">$current_date{'serial'} <a href="/cgi-bin/frame.cgi?date=$current_photo">Permanent Link</a></div>
		$current_caption
	</div>

	<div id="previous-photos">
		$left_arrow
		$archive[0]
		$archive[1]
		$archive[2]
		$archive[3]
		$archive[4]
		<div id="arrow-right-blank"></div>
	</div>

<!-- End output of photoblog.cgi -->

END_OF_HTML
;