#!/usr/bin/perl -w

# upgrade.cgi
#
# This script is part of Birch. It updates the photos.xml file to the new format.
#
# http://yourservername.com/cgi-bin/upgrade.cgi
#
# Copyright (c) 2006, Ross Harmes
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
# 
#	*	Redistributions of source code must retain the above copyright notice, 
#		this list of conditions and the following disclaimer.
#	*	Redistributions in binary form must reproduce the above copyright notice, 
#		this list of conditions and the following disclaimer in the documentation 
#		and/or other materials provided with the distribution.
#	*	Neither the name of View Software nor the names of its contributors may be 
#		used to endorse or promote products derived from this software without 
#		specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Changelog: 
#	2005.12.11: File created.

use strict;
use CGI; 

my $version = "1.3"; 				# Set this to the current version number.
my $photo_list = "photos.xml";
my ($lines_read, $line);

# Read the photo list and modify each line with width and height tags.

open LIST, "$photo_list";
while(<LIST>) { 
	$line = $_;
	
	if($line =~ /<date>.*?<\/date><caption>.*?<\/caption><orientation>.*?<\/orientation><width>.*?<\/width><height>.*?<\/height>/) {
		$lines_read .= $line;	# Do nothing, this line has already been updated.
		}
	elsif($line =~ /<date>(.*?)<\/date><caption>(.*?)<\/caption><orientation>(.*?)<\/orientation>/) {
	
		# This line needs to be updated.
		
		if($3 eq 'horizontal') {
			$lines_read .= "\t<post><date>" . $1 . "<\/date><caption>" . $2 . "<\/caption><orientation>" . $3 . "<\/orientation><width>600<\/width><height>450<\/height></post>\n"
			}
		else {
			$lines_read .= "\t<post><date>" . $1 . "<\/date><caption>" . $2 . "<\/caption><orientation>" . $3 . "<\/orientation><width>450<\/width><height>600<\/height></post>\n"
			}
		}
	else {
		$lines_read .= $line;	# Do nothing, this is not a post entry.
		}
	}
close LIST;
	
# Write the lines back into the file.

open LIST, ">$photo_list";	
print LIST $lines_read;
close LIST; 

# Output a message saying the upgrade is complete.

print <<END_OF_HTML

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd  ">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"> 
<head>
	<title>birch administration</title>

	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
		
	<link rel="stylesheet" type="text/css" href="/includes/admin.css" /> 
	
</head>
<body>

<div id="content">

	<h1 id="title">Birch Administration :: <a href="/">Main</a></h1>
	
	<div id="upgrade" class="section">
	
		<div id="upgrade-1">

			<strong>Upgrade Complete</strong>
			
			<p>
				The upgrade completed successfully. You can now enjoy the new features
				of version $version. From here, you can go to <a href="/">the main page</a>
				or to <a href="/admin/">the administration page</a>. Thank you for using <a href="http://view-sw.com/birch/">Birch $version</a>
			</p>
	
		</div>
		
	</div>

</div>

</body>
</html>

END_OF_HTML
;
