#!/usr/bin/perl -w

# install/index.cgi
#
# This script is part of Birch. It creates the .htpasswd file needed to give the admin directory
# password protection, as ensures that all the other necessary files are installed properly.
#
# Copyright (c) 2006, Ross Harmes
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
# 
#	*	Redistributions of source code must retain the above copyright notice, 
#		this list of conditions and the following disclaimer.
#	*	Redistributions in binary form must reproduce the above copyright notice, 
#		this list of conditions and the following disclaimer in the documentation 
#		and/or other materials provided with the distribution.
#	*	Neither the name of View Software nor the names of its contributors may be 
#		used to endorse or promote products derived from this software without 
#		specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Changelog: 
#	2005.02.01: File created.
#	2005.02.03: Added section to create thumbnails and photos directories.

use CGI;
use strict;

my $q = new CGI; 

my $htpasswd_file 	= "../admin/.htpasswd";

my $sidebar_file 	= "../includes/sidebar.html";
my $title_file 		= "../includes/title.html";
my $author_file		= "../includes/author.html";

my $admin_htaccess	= "../admin/.htaccess";
my $base_htaccess	= "../.htaccess";
my $bin_htaccess	= "../cgi-bin/.htaccess";
my $archives_htaccess = "../archives/.htaccess";
my $rss_htaccess	= "../rss/.htaccess";

my $thumb_dir		= "../thumbnails";
my $photo_dir		= "../photos";

my $message = "<p>" .
			  "Most of the needed files are already in place after decompressing the zip file. If this page " .
			  "loads then the .htaccess files are being read. The final step is to enter information specific " .
			  "to your photoblog." .
			  "</p>" .
			  "<p>" .
			  "The username and password are the ones that you will use to access the administration page. The " .
			  "two password fields must match." .
			  "</p>" .
			  "<p>" .
			  "The photoblog name is what will appear at the top of each browser window and as the main title for " .
			  "your site. The tagline is an optional sentence that appears under the title. The photographers' name " .
			  "is what appears in the copyright statement at the bottom of each page." .
			  "</p>";
			  
my $button_value = "Install Birch";
			  
my $username 	= "";
my $password 	= "";

my $page_title 	= "";
my $tagline 	= "";
my $fullname 	= "";

my $install	= $q->param("install");

if($install ne "") {

	# Get the rest of the HTTP vars.

	$username 	= $q->param("username");
	$password 	= $q->param("password1");

	$page_title = $q->param("photoblog-name");
	$tagline 	= $q->param("tagline");
	$fullname 	= $q->param("fullname");
	
    # Set the permissons on all of the executable files.
    
    chmod(0755, "../admin/index.cgi", "../admin/.htaccess", "../cgi-bin/Birch.pm",  "../cgi-bin/archives.cgi",  "../cgi-bin/edit_post.cgi",  "../cgi-bin/frame.cgi");
    chmod(0755, "../cgi-bin/htpasswd.pl",  "../cgi-bin/options.cgi",  "../cgi-bin/photoblog.cgi",  "../cgi-bin/",  "../cgi-bin/upload.cgi");
    chmod(0755, "../archives/index.cgi", "../rss/index.cgi");
    
    # Create the directories where the photos are put.
    
    `mkdir $thumb_dir`;
    `mkdir $photo_dir`;
    chmod(0755, $thumb_dir, $photo_dir);

	# Create .htpasswd file.
	
	chdir("../admin/");
	my $admin_dir = `pwd`;
	$admin_dir =~ s/\n//;
	`../cgi-bin/htpasswd.pl --userid $username --passwd $password --writeover --dir ../admin/`;
	
	# Create the .htaccess files.
	
	my $htaccess_string = 	"Options Includes +ExecCGI\n\n" .
							"AuthName \"Birch Administration Login\"\n" .
							"AuthType Basic\n" .
							"AuthUserFile $admin_dir/.htpasswd\n\n" .
							"require valid-user\n";
	
	open(HTACCESS, ">$admin_htaccess");
    print HTACCESS $htaccess_string;
    close(HTACCESS);
    
    $htaccess_string = 	"AddHandler server-parsed .html\n" .
						"Options Includes\n";
	
	open(HTACCESS, ">$base_htaccess");
    print HTACCESS $htaccess_string;
    close(HTACCESS);
    
    $htaccess_string = "Options Includes +ExecCGI\n";
    
    open(HTACCESS, ">$bin_htaccess");
    print HTACCESS $htaccess_string;
    close(HTACCESS);
    
    open(HTACCESS, ">$archives_htaccess");
    print HTACCESS $htaccess_string;
    close(HTACCESS);
    
    open(HTACCESS, ">$rss_htaccess");
    print HTACCESS $htaccess_string;
    close(HTACCESS);
    
    # Insert the page/author info into the included sidebar file.
    
    my $sidebar_string = "\t<h1 id=\"page-title\">\n\t\t" . $page_title . "\n\t</h1>\n\n" .
						 "\t<h2 id=\"tagline\">\n\t\t" . $tagline . "\n\t</h2>\n\n" .
						 "\t<div id=\"nav\">\n\t\t<a href=\"/\">Home</a>\n\t\t<a href=\"/archives/\">Archives</a>\n\t\t<a href=\"/colophon.html\">Colophon</a>\n\t\t<a href=\"/rss/\">RSS</a>\n\t</div>\n\n" .
						 "\t<div id=\"badges\">\n\t\t<a href=\"http://www.view-sw.com/birch/\" id=\"powered-by-birch\">Powered By Birch</a>\n\t</div>\n\n" .
						 "\t<div id=\"copyright\">\n\t\t&copy; 2005 " . $fullname . "\n\t</div>";
    
    open(SIDEBAR, ">$sidebar_file");
    print SIDEBAR $sidebar_string;
    close(SIDEBAR);
    
    # Insert the author into the HTML fragment.
    
    open(AUTHOR, ">$author_file");
    print AUTHOR $fullname;
    close(AUTHOR);
    
    # Insert the page title into the HTML fragment.
    
    open(TITLE, ">$title_file");
    print TITLE $page_title;
    close(TITLE);
    
    $message =  "<p>" .
    			"The installation was successful. You can now login to the <a href=\"/admin/\">administration page</a> ".
    			"with your username and password and start uploading photos. Or you can go to the <a href=\"/\">main page</a> " .
    			"and see how the title and tagline look. If you don't like it, you can come back here and change it. " .
    			"</p>" .
    			"<p>" .
    			"Once you are happy with the title and tagline, <span style=\"color : #000;\">YOU MUST DELETE THE 'install' DIRECTORY</span>. " .
    			"If this isn't done, a malicious user can change your login information.".
    			"</p>" .
    			"<p>" .
    			"Enjoy, and thanks for installing Birch." .
    			"</p>";
    			
    $button_value = "Change Options";
	}

# The first time it is run, the following page is displayed:

print "Content-type: text/html\n\n";
print <<END_OF_HTML
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd  ">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"> 
<head>
	<title>Birch Install</title>

	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />

	<link rel="stylesheet" type="text/css" href="/includes/admin.css" />
	<script type="text/javascript" src="/includes/admin.js"></script> 
	
</head>
<body>

<div id="content">

	<h1 id="title">Birch Install</h1>

	<div id="install" class="section">
	
		<form action="/install/" method="post" onsubmit="return checkPasswords('password1', 'password2');">
		
			<div id="install-main">
			
				<strong>User Information</strong>

				<label for="username">Username:</label>
				<input type="text" name="username" id="username" value="$username" />
				
				<label for="password1">Password:</label>
				<input type="password" name="password1" id="password1" />
				
				<label for="password2">Repeat Password:</label>
				<input type="password" name="password2" id="password2" />

				<br />
				<strong>Page Information</strong>

				<label for="photoblog-name">Photoblog Name:</label>
				<input type="text" name="photoblog-name" id="photoblog-name" value="$page_title" />
				
				<label for="tagline">Tagline (Optional):</label>
				<input type="text" name="tagline" id="tagline" value="$tagline" />
				
				<label for="fullname">Photographer's Name:</label>
				<input type="text" name="fullname" id="fullname" value="$fullname" />

			</div>
	
			<div id="install-side">

				<strong>Installation Instructions:</strong>
				$message
				<input type="submit" name="install" value="$button_value" id="install-submit" />
			
			</div>
		
		</form>
		
		<br class="clear" />
		
	</div>

</div>

</body>
</html>

END_OF_HTML
;

exit;